@@ -1,20 +1,16 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
-
-  This software is provided AS-IS with no warranty, either express or
-  implied.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
- */
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/* $Id: gdevdjet.c 4656 2004-01-29 18:19:41Z ray $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevdjet.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* HP LaserJet/DeskJet driver for Ghostscript */
 #include "gdevprn.h"
 #include "gdevdljm.h"
@@ -111,94 +107,113 @@ private dev_proc_print_page_copies(ljet4_print_page_copies);
 private dev_proc_print_page_copies(ljet4d_print_page_copies);
 private dev_proc_print_page_copies(lp2563_print_page_copies);
 private dev_proc_print_page_copies(oce9050_print_page_copies);
+private dev_proc_get_params(hpjet_get_params);
+private dev_proc_put_params(hpjet_put_params);
 
 private const gx_device_procs prn_hp_procs =
 prn_params_procs(hpjet_open, gdev_prn_output_page, hpjet_close,
-		 gdev_prn_get_params, gdev_prn_put_params);
+		 hpjet_get_params, hpjet_put_params);
 
-const gx_device_printer gs_deskjet_device =
-prn_device_copies(prn_hp_procs, "deskjet",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0, 0, 0, 0,		/* margins filled in by hpjet_open */
-		  1, djet_print_page_copies);
+typedef struct gx_device_hpjet_s gx_device_hpjet;
 
-const gx_device_printer gs_djet500_device =
-prn_device_copies(prn_hp_procs, "djet500",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0, 0, 0, 0,		/* margins filled in by hpjet_open */
-		  1, djet500_print_page_copies);
+struct gx_device_hpjet_s {
+    gx_device_common;
+    gx_prn_device_common;
+    int MediaPosition;
+    bool MediaPosition_set;
+    bool ManualFeed;
+    bool ManualFeed_set;
+};
 
-const gx_device_printer gs_fs600_device =
-prn_device_copies(prn_hp_procs, "fs600",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI2, Y_DPI2,
-		  0.23, 0.0, 0.23, 0.04,      /* margins */
-		  1, fs600_print_page_copies);
+#define HPJET_DEVICE(procs, dname, w10, h10, xdpi, ydpi, lm, bm, rm, tm, color_bits, print_page_copies)\
+  { prn_device_std_margins_body_copies(gx_device_hpjet, procs, dname, \
+        w10, h10, xdpi, ydpi, lm, tm, lm, bm, rm, tm, color_bits, \
+        print_page_copies), \
+    0, false, false, false }
 
-const gx_device_printer gs_laserjet_device =
-prn_device_copies(prn_hp_procs, "laserjet",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0.05, 0.25, 0.55, 0.25,	/* margins */
-		  1, ljet_print_page_copies);
+const gx_device_hpjet gs_deskjet_device =
+HPJET_DEVICE(prn_hp_procs, "deskjet",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0, 0, 0, 0,		/* margins filled in by hpjet_open */
+	     1, djet_print_page_copies);
 
-const gx_device_printer gs_ljetplus_device =
-prn_device_copies(prn_hp_procs, "ljetplus",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0.05, 0.25, 0.55, 0.25,	/* margins */
-		  1, ljetplus_print_page_copies);
+const gx_device_hpjet gs_djet500_device =
+HPJET_DEVICE(prn_hp_procs, "djet500",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0, 0, 0, 0,		/* margins filled in by hpjet_open */
+	     1, djet500_print_page_copies);
 
-const gx_device_printer gs_ljet2p_device =
-prn_device_copies(prn_hp_procs, "ljet2p",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0.20, 0.25, 0.25, 0.25,	/* margins */
-		  1, ljet2p_print_page_copies);
+const gx_device_hpjet gs_fs600_device =
+HPJET_DEVICE(prn_hp_procs, "fs600",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI2, Y_DPI2,
+	     0.23, 0.0, 0.23, 0.04,      /* margins */
+	     1, fs600_print_page_copies);
 
-const gx_device_printer gs_ljet3_device =
-prn_device_copies(prn_hp_procs, "ljet3",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0.20, 0.25, 0.25, 0.25,	/* margins */
-		  1, ljet3_print_page_copies);
+const gx_device_hpjet gs_laserjet_device =
+HPJET_DEVICE(prn_hp_procs, "laserjet",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0.05, 0.25, 0.55, 0.25,	/* margins */
+	     1, ljet_print_page_copies);
 
-const gx_device_printer gs_ljet3d_device =
-prn_device_copies(prn_hp_procs, "ljet3d",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0.20, 0.25, 0.25, 0.25,	/* margins */
-		  1, ljet3d_print_page_copies);
+const gx_device_hpjet gs_ljetplus_device =
+HPJET_DEVICE(prn_hp_procs, "ljetplus",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0.05, 0.25, 0.55, 0.25,	/* margins */
+	     1, ljetplus_print_page_copies);
 
-const gx_device_printer gs_ljet4_device =
-prn_device_copies(prn_hp_procs, "ljet4",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI2, Y_DPI2,
-		  0, 0, 0, 0,		/* margins */
-		  1, ljet4_print_page_copies);
+const gx_device_hpjet gs_ljet2p_device =
+HPJET_DEVICE(prn_hp_procs, "ljet2p",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0.20, 0.25, 0.25, 0.25,	/* margins */
+	     1, ljet2p_print_page_copies);
 
-const gx_device_printer gs_ljet4d_device =
-prn_device_copies(prn_hp_procs, "ljet4d",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI2, Y_DPI2,
-		  0, 0, 0, 0,		/* margins */
-		  1, ljet4d_print_page_copies);
+const gx_device_hpjet gs_ljet3_device =
+HPJET_DEVICE(prn_hp_procs, "ljet3",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0.20, 0.25, 0.25, 0.25,	/* margins */
+	     1, ljet3_print_page_copies);
 
-const gx_device_printer gs_lp2563_device =
-prn_device_copies(prn_hp_procs, "lp2563",
-		  DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
-		  X_DPI, Y_DPI,
-		  0, 0, 0, 0,		/* margins */
-		  1, lp2563_print_page_copies);
+const gx_device_hpjet gs_ljet3d_device =
+HPJET_DEVICE(prn_hp_procs, "ljet3d",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0.20, 0.25, 0.25, 0.25,	/* margins */
+	     1, ljet3d_print_page_copies);
 
-const gx_device_printer gs_oce9050_device =
-prn_device_copies(prn_hp_procs, "oce9050",
-		  24 * 10, 24 * 10,	/* 24 inch roll (can print 32" also) */
-		  400, 400,		/* 400 dpi */
-		  0, 0, 0, 0,		/* margins */
-		  1, oce9050_print_page_copies);
+const gx_device_hpjet gs_ljet4_device =
+HPJET_DEVICE(prn_hp_procs, "ljet4",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI2, Y_DPI2,
+	     0, 0, 0, 0,		/* margins */
+	     1, ljet4_print_page_copies);
+
+const gx_device_hpjet gs_ljet4d_device =
+HPJET_DEVICE(prn_hp_procs, "ljet4d",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI2, Y_DPI2,
+	     0, 0, 0, 0,		/* margins */
+	     1, ljet4d_print_page_copies);
+
+const gx_device_hpjet gs_lp2563_device =
+HPJET_DEVICE(prn_hp_procs, "lp2563",
+	     DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
+	     X_DPI, Y_DPI,
+	     0, 0, 0, 0,		/* margins */
+	     1, lp2563_print_page_copies);
+
+const gx_device_hpjet gs_oce9050_device =
+HPJET_DEVICE(prn_hp_procs, "oce9050",
+	     24 * 10, 24 * 10,	/* 24 inch roll (can print 32" also) */
+	     400, 400,		/* 400 dpi */
+	     0, 0, 0, 0,		/* margins */
+	     1, oce9050_print_page_copies);
 
 /* Open the printer, adjusting the margins if necessary. */
 private int
@@ -263,23 +278,47 @@ hpjet_close(gx_device * pdev)
 
 /* ------ Internal routines ------ */
 
+/* Make an init string that contains paper tray selection. The resulting
+   init string is stored in buf, so make sure that buf is at least 5
+   bytes larger than str. */
+private void
+hpjet_make_init(gx_device_printer *pdev, char *buf, const char *str)
+{
+    gx_device_hpjet *dev = (gx_device_hpjet *)pdev;
+    int paper_source = -1;
+    int paper_source_tab[] = { 5, 1 };
+
+    if (dev->ManualFeed_set && dev->ManualFeed) paper_source = 2;
+    else if (dev->MediaPosition_set && dev->MediaPosition >= 0 &&
+	     dev->MediaPosition < countof(paper_source_tab))
+	paper_source = paper_source_tab[dev->MediaPosition];
+    if (paper_source >= 0)
+	sprintf(buf, "%s\033&l%dH", str, paper_source);
+    else
+	sprintf(buf, "%s", str);
+}
+
 /* The DeskJet can compress (mode 2) */
 private int
 djet_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 		       int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033&k1W\033*b2M");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_DJ_FEATURES,
-					"\033&k1W\033*b2M");
+					300, PCL_DJ_FEATURES, init);
 }
 /* The DeskJet500 can compress (modes 2&3) */
 private int
 djet500_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			  int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033&k1W");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_DJ500_FEATURES,
-					"\033&k1W");
+					300, PCL_DJ500_FEATURES, init);
 }
 /* The Kyocera FS-600 laser printer (and perhaps other printers */
 /* which use the PeerlessPrint5 firmware) doesn't handle        */
@@ -289,30 +328,36 @@ fs600_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			int num_copies)
 {
     int dots_per_inch = (int)pdev->y_pixels_per_inch;
-    char real_init[60];
+    char base_init[60];
+    char init[80];
 
-    sprintf(real_init, "\033*r0F\033&u%dD", dots_per_inch);
+    sprintf(base_init, "\033*r0F\033&u%dD", dots_per_inch);
+    hpjet_make_init(pdev, init, base_init);
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
 					dots_per_inch, PCL_FS600_FEATURES,
-					real_init);
+					init);
 }
 /* The LaserJet series II can't compress */
 private int
 ljet_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 		       int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033*b0M");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_LJ_FEATURES,
-					"\033*b0M");
+					300, PCL_LJ_FEATURES, init);
 }
 /* The LaserJet Plus can't compress */
 private int
 ljetplus_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			   int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033*b0M");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_LJplus_FEATURES,
-					"\033*b0M");
+					300, PCL_LJplus_FEATURES, init);
 }
 /* LaserJet series IIp & IId compress (mode 2) */
 /* but don't support *p+ or *b vertical spacing. */
@@ -320,9 +365,11 @@ private int
 ljet2p_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			 int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033*r0F\033*b2M");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_LJ2p_FEATURES,
-					"\033*r0F\033*b2M");
+					300, PCL_LJ2p_FEATURES, init);
 }
 /* All LaserJet series IIIs (III,IIId,IIIp,IIIsi) compress (modes 2&3) */
 /* They also need their coordinate system translated slightly. */
@@ -330,18 +377,22 @@ private int
 ljet3_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033&l-180u36Z\033*r0F");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_LJ3_FEATURES,
-					"\033&l-180u36Z\033*r0F");
+					300, PCL_LJ3_FEATURES, init);
 }
 /* LaserJet IIId is same as LaserJet III, except for duplex */
 private int
 ljet3d_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			 int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033&l-180u36Z\033*r0F");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_LJ3D_FEATURES,
-					"\033&l-180u36Z\033*r0F");
+					300, PCL_LJ3D_FEATURES, init);
 }
 /* LaserJet 4 series compresses, and it needs a special sequence to */
 /* allow it to specify coordinates at 600 dpi. */
@@ -351,24 +402,28 @@ ljet4_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			int num_copies)
 {
     int dots_per_inch = (int)pdev->y_pixels_per_inch;
-    char real_init[60];
+    char base_init[60];
+    char init[80];
 
-    sprintf(real_init, "\033&l-180u36Z\033*r0F\033&u%dD", dots_per_inch);
+    sprintf(base_init, "\033&l-180u36Z\033*r0F\033&u%dD", dots_per_inch);
+    hpjet_make_init(pdev, init, base_init);
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
 					dots_per_inch, PCL_LJ4_FEATURES,
-					real_init);
+					init);
 }
 private int
 ljet4d_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			 int num_copies)
 {
     int dots_per_inch = (int)pdev->y_pixels_per_inch;
-    char real_init[60];
+    char base_init[60];
+    char init[80];
 
-    sprintf(real_init, "\033&l-180u36Z\033*r0F\033&u%dD", dots_per_inch);
+    sprintf(base_init, "\033&l-180u36Z\033*r0F\033&u%dD", dots_per_inch);
+    hpjet_make_init(pdev, init, base_init);
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
 					dots_per_inch, PCL_LJ4D_FEATURES,
-					real_init);
+					init);
 }
 /* The 2563B line printer can't compress */
 /* and doesn't support *p+ or *b vertical spacing. */
@@ -376,9 +431,11 @@ private int
 lp2563_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			 int num_copies)
 {
+    char init[80];
+
+    hpjet_make_init(pdev, init, "\033*b0M");
     return dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					300, PCL_LP2563B_FEATURES,
-					"\033*b0M");
+					300, PCL_LP2563B_FEATURES, init);
 }
 /* The Oce line printer has TIFF compression */
 /* and doesn't support *p+ or *b vertical spacing. */
@@ -387,6 +444,7 @@ oce9050_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
 			  int num_copies)
 {
     int code;
+    char init[80];
 
     /* Switch to HP_RTL. */
     fputs("\033%1B", prn_stream);	/* Enter HPGL/2 mode */
@@ -394,9 +452,10 @@ oce9050_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
     fputs("IN;", prn_stream);	/* Initialize (start plot) */
     fputs("\033%1A", prn_stream);	/* Enter PCL mode */
 
+    hpjet_make_init(pdev, init, "\033*b0M");
+
     code = dljet_mono_print_page_copies(pdev, prn_stream, num_copies,
-					400, PCL_OCE9050_FEATURES,
-					"\033*b3M");
+					400, PCL_OCE9050_FEATURES, init);
 
     /* Return to HPGL/2 mode. */
     fputs("\033%1B", prn_stream);	/* Enter HPGL/2 mode */
@@ -408,3 +467,53 @@ oce9050_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
     }
     return code;
 }
+
+private int
+hpjet_get_params(gx_device *pdev, gs_param_list *plist)
+{
+    gx_device_hpjet *dev = (gx_device_hpjet *)pdev;
+    int code = gdev_prn_get_params(pdev, plist);
+
+    if (code >= 0)
+	code = param_write_bool(plist, "ManualFeed", &dev->ManualFeed);
+    return code;
+}
+
+private int
+hpjet_put_params(gx_device *pdev, gs_param_list *plist)
+{
+    gx_device_hpjet *dev = (gx_device_hpjet *)pdev;
+    int code;
+    bool ManualFeed;
+    bool ManualFeed_set = false;
+    int MediaPosition;
+    bool MediaPosition_set = false;
+
+    code = param_read_bool(plist, "ManualFeed", &ManualFeed);
+    if (code == 0) ManualFeed_set = true;
+    if (code >= 0) {
+	code = param_read_int(plist, "%MediaSource", &MediaPosition);
+	if (code == 0) MediaPosition_set = true;
+	else if (code < 0) {
+	    if (param_read_null(plist, "%MediaSource") == 0) {
+		code = 0;
+	    }
+	}
+    }
+
+    if (code >= 0)
+	code = gdev_prn_put_params(pdev, plist);
+
+    if (code >= 0) {
+	if (ManualFeed_set) {
+	    dev->ManualFeed = ManualFeed;
+	    dev->ManualFeed_set = true;
+	}
+	if (MediaPosition_set) {
+	    dev->MediaPosition = MediaPosition;
+	    dev->MediaPosition_set = true;
+	}
+    }
+
+    return code;
+}
