@@ -1,20 +1,16 @@
-/* Copyright (C) 1989, 1995, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/* $Id: gdevmem.c 5046 2004-05-26 04:11:01Z dan $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevmem.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Generic "memory" (stored bitmap) device */
 #include "memory_.h"
 #include "gx.h"
@@ -26,6 +22,7 @@
 #include "gxgetbit.h"
 #include "gxdevmem.h"		/* semi-public definitions */
 #include "gdevmem.h"		/* private definitions */
+#include "gstrans.h"
 
 /* Structure descriptor */
 public_st_device_memory();
@@ -246,19 +243,37 @@ gdev_mem_data_size(const gx_device_memory * dev, int width, int height)
  * compute the maximum height.
  */
 int
-gdev_mem_max_height(const gx_device_memory * dev, int width, ulong size)
+gdev_mem_max_height(const gx_device_memory * dev, int width, ulong size,
+		bool page_uses_transparency)
 {
-    ulong max_height = size /
-	(bitmap_raster(width * dev->color_info.depth) +
-	 sizeof(byte *) * max(dev->num_planes, 1));
-    int height = (int)min(max_height, max_int);
+    int height;
+    ulong max_height;
 
-    /*
-     * Because of alignment rounding, the just-computed height might
-     * be too large by a small amount.  Adjust it the easy way.
-     */
-    while (gdev_mem_data_size(dev, width, height) > size)
-	--height;
+    if (page_uses_transparency) {
+        /*
+         * If the device is using PDF 1.4 transparency then we will need to
+         * also allocate image buffers for doing the blending operations.
+         * We can only estimate the space requirements.  However since it
+	 * is only an estimate, we may exceed our desired buffer space while
+	 * processing the file.
+	 */
+        max_height = size / (bitmap_raster(width
+		* dev->color_info.depth + ESTIMATED_PDF14_ROW_SPACE(width))
+		+ sizeof(byte *) * max(dev->num_planes, 1));
+        height = (int)min(max_height, max_int);
+    } else {
+	/* For non PDF 1.4 transparency, we can do an exact calculation */
+        max_height = size /
+	    (bitmap_raster(width * dev->color_info.depth) +
+	     sizeof(byte *) * max(dev->num_planes, 1));
+        height = (int)min(max_height, max_int);
+        /*
+         * Because of alignment rounding, the just-computed height might
+         * be too large by a small amount.  Adjust it the easy way.
+         */
+        while (gdev_mem_data_size(dev, width, height) > size)
+	    --height;
+    }
     return height;
 }
 
