@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfo.c 5082 2004-06-08 11:43:04Z igor $ */
+/* $Id: gdevpdfo.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Cos object support */
 #include "memory_.h"
 #include "string_.h"
@@ -293,8 +290,9 @@ cos_value_free(const cos_value_t *pcv, const cos_object_t *pco,
 	break;
     case COS_VALUE_OBJECT:
 	/* Free the object if this is the only reference to it. */
-	if (!pcv->contents.object->id)
-	    cos_free(pcv->contents.object, cname);
+	if (pcv->contents.object != NULL) /* see cos_dict_objects_delete. */
+	    if (!pcv->contents.object->id)
+		cos_free(pcv->contents.object, cname);
     case COS_VALUE_RESOURCE:
 	break;
     }
@@ -315,9 +313,8 @@ cos_value_write_spaced(const cos_value_t *pcv, gx_device_pdf *pdev,
 	    case '/': case '(': case '<': break;
 	    default: stream_putc(s, ' ');
 	    }
-	pdf_write_value(pdev, pcv->contents.chars.data,
+	return pdf_write_value(pdev, pcv->contents.chars.data,
 			pcv->contents.chars.size, object_id);
-	break;
     case COS_VALUE_RESOURCE:
 	pprintld1(s, "/R%ld", pcv->contents.object->id);
 	break;
@@ -523,7 +520,7 @@ cos_array_write(const cos_object_t *pco, gx_device_pdf *pdev, gs_id object_id)
 	    stream_putc(s, '\n');
 	for (; pcae->index > last_index; ++last_index)
 	    stream_puts(s, "null\n");
-	cos_value_write(&pcae->value, pdev);
+	cos_value_write_spaced(&pcae->value, pdev, false, object_id);
     }
     DISCARD(cos_array_reorder(pca, first));
     stream_puts(s, "]");
@@ -637,6 +634,7 @@ cos_array_add_real(cos_array_t *pca, floatp r)
     stream s;
     cos_value_t v;
 
+    s_init(&s, NULL);
     swrite_string(&s, str, sizeof(str));
     pprintg1(&s, "%g", r);
     return cos_array_add(pca, cos_string_value(&v, str, stell(&s)));
@@ -804,7 +802,8 @@ cos_dict_objects_write(const cos_dict_t *pcd, gx_device_pdf *pdev)
 
     for (; pcde; pcde = pcde->next)
 	if (COS_VALUE_IS_OBJECT(&pcde->value) &&
-	    pcde->value.contents.object->id)
+	    pcde->value.contents.object->id  &&
+	    !pcde->value.contents.object->written /* ForOPDFRead only. */)
 	    cos_write_object(pcde->value.contents.object, pdev);
     return 0;
 }
@@ -814,11 +813,20 @@ cos_dict_objects_delete(cos_dict_t *pcd)
     cos_dict_element_t *pcde = pcd->elements;
 
     /*
+     * Delete duplicate references to prevent a dual object freeing.
      * Delete the objects' IDs so that freeing the dictionary will
      * free them.
      */
-    for (; pcde; pcde = pcde->next)
-	pcde->value.contents.object->id = 0;
+    for (; pcde; pcde = pcde->next) {
+	if (pcde->value.contents.object) {
+	    cos_dict_element_t *pcde1 = pcde->next;
+	
+	    for (; pcde1; pcde1 = pcde1->next)
+		if (pcde->value.contents.object == pcde1->value.contents.object)
+		    pcde1->value.contents.object = NULL;
+	    pcde->value.contents.object->id = 0;
+	}
+    }	
     return 0;
 }
 
@@ -844,6 +852,17 @@ cos_dict_put_copy(cos_dict_t *pcd, const byte *key_data, uint key_size,
 	ppcde = &next->next;
     if (next) {
 	/* We're replacing an existing element. */
+	if ((pvalue->value_type == COS_VALUE_SCALAR ||
+	     pvalue->value_type == COS_VALUE_CONST) &&
+	    pvalue->value_type == next->value.value_type &&
+	    !bytes_compare(pvalue->contents.chars.data, pvalue->contents.chars.size, 
+		next->value.contents.chars.data, next->value.contents.chars.size))
+	    return 0; /* Same as old value. */
+	if ((pvalue->value_type == COS_VALUE_OBJECT ||
+	     pvalue->value_type == COS_VALUE_RESOURCE) &&
+	    pvalue->value_type == next->value.value_type &&
+	    pvalue->contents.object == next->value.contents.object)
+	    return 0; /* Same as old value. */
 	code = cos_copy_element_value(&value, mem, pvalue,
 				      (flags & DICT_COPY_VALUE) != 0);
 	if (code < 0)
@@ -938,6 +957,7 @@ cos_dict_put_c_key_real(cos_dict_t *pcd, const char *key, floatp value)
     byte str[50];		/****** ADHOC ******/
     stream s;
 
+    s_init(&s, NULL);
     swrite_string(&s, str, sizeof(str));
     pprintg1(&s, "%g", value);
     return cos_dict_put_c_key_string(pcd, key, str, stell(&s));
@@ -1056,6 +1076,22 @@ cos_dict_equal(const cos_object_t *pco0, const cos_object_t *pco1, gx_device_pdf
     return true;
 }
 
+/* Process all entries in a dictionary. */
+int
+cos_dict_forall(const cos_dict_t *pcd, void *client_data, 
+		int (*proc)(void *client_data, const byte *key_data, uint key_size, const cos_value_t *v))
+{
+    cos_dict_element_t *pcde = pcd->elements;
+
+    for (; pcde; pcde = pcde->next) {
+	int code = proc(client_data, pcde->key.data, pcde->key.size, &pcde->value);
+
+	if (code != 0)
+	    return code;
+    }
+    return 0;
+}
+
 /* Set up a parameter list that writes into a Cos dictionary. */
 
 /* We'll implement the other printers later if we have to. */
@@ -1092,7 +1128,8 @@ cos_param_put_typed(gs_param_list * plist, gs_param_name pkey,
 	int len, skip;
 	byte *str;
 
-	ppp = param_printer_params_default;
+	s_init(&s, NULL);
+        ppp = param_printer_params_default;
 	ppp.prefix = ppp.suffix = ppp.item_prefix = ppp.item_suffix = 0;
 	ppp.print_ok = pclist->print_ok;
 	s_init_param_printer(&pplist, &ppp, &s);
