@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtd.h 3813 2003-04-11 17:54:39Z igor $ */
+/* $Id: gdevpdtd.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* FontDescriptor structure and API for pdfwrite */
 
 #ifndef gdevpdtd_INCLUDED
@@ -133,10 +130,24 @@ int pdf_compute_font_descriptor(pdf_font_descriptor_t *pfd);
 int pdf_finish_FontDescriptor(gx_device_pdf *pdev,
 			      pdf_font_descriptor_t *pfd);
 
+int pdf_finish_font_descriptors(gx_device_pdf *pdev,
+			int (*finish_proc)(gx_device_pdf *,
+					   pdf_font_descriptor_t *));
 /*
  * Write a FontDescriptor.
  */
 int pdf_write_FontDescriptor(gx_device_pdf *pdev,
 			     pdf_font_descriptor_t *pfd);
 
+/*
+ * Release a FontDescriptor components.
+ */
+int pdf_release_FontDescriptor_components(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd);
+
+/*
+ * Mark a FontDescriptor used in a text.
+ */
+int pdf_mark_font_descriptor_used(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd);
+
+
 #endif /* gdevpdtd_INCLUDED */
