@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdti.c 5479 2004-11-01 21:09:08Z igor $ */
+/* $Id: gdevpdti.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Bitmap font implementation for pdfwrite */
 #include "memory_.h"
 #include "string_.h"
@@ -37,19 +34,30 @@
 /*typedef struct pdf_char_proc_s pdf_char_proc_t;*/  /* gdevpdfx.h */
 struct pdf_char_proc_s {
     pdf_resource_common(pdf_char_proc_t);
-    pdf_font_resource_t *font;
-    pdf_char_proc_t *char_next;	/* next char_proc for same font */
+    pdf_char_proc_ownership_t *owner_fonts; /* fonts using this charproc. */
     int y_offset;		/* of character (0,0) */
-    gs_char char_code;
-    gs_const_string char_name;
     gs_point real_width;        /* Not used with synthesised bitmap fonts. */
     gs_point v;			/* Not used with synthesised bitmap fonts. */
 };
 
 /* The descriptor is public for pdf_resource_type_structs. */
-gs_public_st_suffix_add2_string1(st_pdf_char_proc, pdf_char_proc_t,
+gs_public_st_suffix_add1(st_pdf_char_proc, pdf_char_proc_t,
   "pdf_char_proc_t", pdf_char_proc_enum_ptrs, pdf_char_proc_reloc_ptrs,
-  st_pdf_resource, font, char_next, char_name);
+  st_pdf_resource, owner_fonts);
+
+struct pdf_char_proc_ownership_s {
+    pdf_char_proc_t *char_proc;
+    pdf_char_proc_ownership_t *font_next;	/* next char_proc for same font */
+    pdf_char_proc_ownership_t *char_next;	/* next char_proc for same charproc */
+    pdf_font_resource_t *font;
+    gs_char char_code;		/* Character code in PDF font. */
+    gs_glyph glyph;		/* Glyph id in Postscript font. */
+    gs_const_string char_name;
+    bool duplicate_char_name;
+};
+gs_private_st_strings1_ptrs4(st_pdf_char_proc_ownership, pdf_char_proc_ownership_t,
+  "pdf_char_proc_ownership_t", pdf_char_proc_ownership_enum_ptrs,
+  pdf_char_proc_ownership_reloc_ptrs, char_name, char_proc, char_next, font_next, font);
 
 /* Define the state structure for tracking bitmap fonts. */
 /*typedef struct pdf_bitmap_fonts_s pdf_bitmap_fonts_t;*/
@@ -71,7 +79,7 @@ pdf_char_proc_id(const pdf_char_proc_t *pcp)
 
 /* Assign a code for a char_proc. */
 private int
-assign_char_code(gx_device_pdf * pdev, int width)
+assign_char_code(gx_device_pdf * pdev, gs_text_enum_t *pte)
 {
     pdf_bitmap_fonts_t *pbfs = pdev->text->bitmap_fonts;
     pdf_font_resource_t *pdfont = pbfs->open_font; /* Type 3 */
@@ -97,6 +105,7 @@ assign_char_code(gx_device_pdf * pdev, int width)
 	pdfont->u.simple.s.type3.FontBBox.p.y = 0;
 	pdfont->u.simple.s.type3.FontBBox.q.x = 1000;
 	pdfont->u.simple.s.type3.FontBBox.q.y = 1000;
+	pdfont->mark_glyph = NULL;
 	gs_make_identity(&pdfont->u.simple.s.type3.FontMatrix);
 	/*
 	 * We "increment" the font name as a radix-26 "number".
@@ -111,18 +120,12 @@ assign_char_code(gx_device_pdf * pdev, int width)
 	pdfont->u.simple.FirstChar = 0;
     }
     c = ++(pdfont->u.simple.LastChar);
-    pdfont->Widths[c] = pdev->char_width.x;
+    pdfont->Widths[c] = psdf_round(pdev->char_width.x, 100, 10); /* See 
+			pdf_write_Widths about rounding. We need to provide 
+			a compatible data for Tj. */
     if (c > pbfs->max_embedded_code)
 	pbfs->max_embedded_code = c;
 
-    /* Synthezise ToUnicode CMap :*/
-    {	gs_text_enum_t *pte = pdev->pte;
-        gs_font *font = pte->current_font;
-
-	code = pdf_add_ToUnicode(pdev, font, pdfont, pte->returned.current_glyph, c); 
-	if (code < 0)
-	    return code;
-    }
     return c;
 }
 
@@ -131,7 +134,7 @@ int
 pdf_write_contents_bitmap(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
 {
     stream *s = pdev->strm;
-    const pdf_char_proc_t *pcp;
+    const pdf_char_proc_ownership_t *pcpo;
     long diff_id = 0;
     int code;
 
@@ -146,15 +149,15 @@ pdf_write_contents_bitmap(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
 	return code;
     stream_puts(s, "/CharProcs <<");
     /* Write real characters. */
-    for (pcp = pdfont->u.simple.s.type3.char_procs; pcp;
-	 pcp = pcp->char_next
+    for (pcpo = pdfont->u.simple.s.type3.char_procs; pcpo;
+	 pcpo = pcpo->char_next
 	 ) {
 	if (pdfont->u.simple.s.type3.bitmap_font)
-	    pprintld2(s, "/a%ld %ld 0 R\n", (long)pcp->char_code,
-		      pdf_char_proc_id(pcp));
-	else {
-	    pdf_put_name(pdev, pcp->char_name.data, pcp->char_name.size);
-	    pprintld1(s, " %ld 0 R\n", pdf_char_proc_id(pcp));
+	    pprintld2(s, "/a%ld %ld 0 R\n", (long)pcpo->char_code,
+		      pdf_char_proc_id(pcpo->char_proc));
+	else if (!pcpo-> duplicate_char_name) {
+	    pdf_put_name(pdev, pcpo->char_name.data, pcpo->char_name.size);
+	    pprintld1(s, " %ld 0 R\n", pdf_char_proc_id(pcpo->char_proc));
 	}
     }
     stream_puts(s, ">>");
@@ -234,28 +237,68 @@ pdf_char_image_y_offset(const gx_device_pdf *pdev, int x, int y, int h)
     return off;
 }
 
-/* Begin a CharProc for a synthesized font. */
+/* Attach a CharProc to a font. */
 private int
-pdf_begin_char_proc_generic(gx_device_pdf * pdev, pdf_font_resource_t *pdfont,
-		    gs_id id, gs_char char_code, 
-		    pdf_char_proc_t ** ppcp, pdf_stream_position_t * ppos)
+pdf_attach_charproc(gx_device_pdf * pdev, pdf_font_resource_t *pdfont, pdf_char_proc_t *pcp,
+		    gs_glyph glyph, gs_char char_code, const gs_const_string *gnstr)
 {
+    pdf_char_proc_ownership_t *pcpo;
+    bool duplicate_char_name = false;
+    
+    for (pcpo = pdfont->u.simple.s.type3.char_procs; pcpo != NULL; pcpo = pcpo->char_next) {
+	if (pcpo->glyph == glyph && pcpo->char_code == char_code)
+	    return 0;
+    }
+    if (!pdfont->u.simple.s.type3.bitmap_font) {
+	for (pcpo = pdfont->u.simple.s.type3.char_procs; pcpo != NULL; pcpo = pcpo->char_next) {
+	    if (!bytes_compare(pcpo->char_name.data, pcpo->char_name.size, gnstr->data, gnstr->size)) {
+		duplicate_char_name = true;
+		break;
+	    }
+	}
+    }
+    pcpo = gs_alloc_struct(pdev->pdf_memory, 
+	    pdf_char_proc_ownership_t, &st_pdf_char_proc_ownership, "pdf_attach_charproc");
+
+    if (pcpo == NULL)
+	return_error(gs_error_VMerror);
+    pcpo->font = pdfont;
+    pcpo->char_next = pdfont->u.simple.s.type3.char_procs;
+    pdfont->u.simple.s.type3.char_procs = pcpo;
+    pcpo->char_proc = pcp;
+    pcpo->font_next = pcp->owner_fonts;
+    pcp->owner_fonts = pcpo;
+    pcpo->char_code = char_code;
+    pcpo->glyph = glyph;
+    if (gnstr == NULL) {
+	pcpo->char_name.data = 0; 
+	pcpo->char_name.size = 0;
+    } else
+	pcpo->char_name = *gnstr;
+    pcpo->duplicate_char_name = duplicate_char_name;
+    return 0;
+}
+
+/* Begin a CharProc for a synthesized (bitmap) font. */
+int
+pdf_begin_char_proc(gx_device_pdf * pdev, int w, int h, int x_width,
+		    int y_offset, gs_id id, pdf_char_proc_t ** ppcp,
+		    pdf_stream_position_t * ppos)
+{
+    int char_code = assign_char_code(pdev, pdev->pte);
+    pdf_bitmap_fonts_t *const pbfs = pdev->text->bitmap_fonts; 
+    pdf_font_resource_t *font = pbfs->open_font; /* Type 3 */
     pdf_resource_t *pres;
     pdf_char_proc_t *pcp;
-    int code;
+    int code = pdf_begin_resource(pdev, resourceCharProc, id, &pres);
 
-    code = pdf_begin_resource(pdev, resourceCharProc, id, &pres);
     if (code < 0)
 	return code;
     pcp = (pdf_char_proc_t *) pres;
-    pcp->font = pdfont;
-    pcp->char_next = pdfont->u.simple.s.type3.char_procs;
-    pdfont->u.simple.s.type3.char_procs = pcp;
-    pcp->char_code = char_code;
+    code = pdf_attach_charproc(pdev, font, pcp, GS_NO_GLYPH, char_code, NULL);
+    if (code < 0)
+	return code;
     pres->object->written = true;
-    pcp->char_name.data = 0; 
-    pcp->char_name.size = 0;
-
     {
 	stream *s = pdev->strm;
 
@@ -272,24 +315,9 @@ pdf_begin_char_proc_generic(gx_device_pdf * pdev, pdf_font_resource_t *pdfont,
     code = pdf_begin_encrypt(pdev, &pdev->strm, pres->object->id);
     if (code < 0)
 	return code;
-    *ppcp = pcp;
-    return 0;
-}
-
-/* Begin a CharProc for a synthesized (bitmap) font. */
-int
-pdf_begin_char_proc(gx_device_pdf * pdev, int w, int h, int x_width,
-		    int y_offset, gs_id id, pdf_char_proc_t ** ppcp,
-		    pdf_stream_position_t * ppos)
-{
-    int char_code = assign_char_code(pdev, x_width);
-    pdf_bitmap_fonts_t *const pbfs = pdev->text->bitmap_fonts; 
-    pdf_font_resource_t *font = pbfs->open_font; /* Type 3 */
-    int code = pdf_begin_char_proc_generic(pdev, font, id, char_code, ppcp, ppos);
-    
     if (code < 0)
 	return code;
-    (*ppcp)->y_offset = y_offset;
+    pcp->y_offset = y_offset;
     font->u.simple.s.type3.FontBBox.p.y =
 	min(font->u.simple.s.type3.FontBBox.p.y, y_offset);
     font->u.simple.s.type3.FontBBox.q.x =
@@ -298,6 +326,7 @@ pdf_begin_char_proc(gx_device_pdf * pdev, int w, int h, int x_width,
 	max(font->u.simple.s.type3.FontBBox.q.y, y_offset + h);
     font->u.simple.s.type3.max_y_offset =
 	max(font->u.simple.s.type3.max_y_offset, h + (h >> 2));
+    *ppcp = pcp;
     return 0;
 }
 
@@ -323,13 +352,40 @@ pdf_end_char_proc(gx_device_pdf * pdev, pdf_stream_position_t * ppos)
     return 0;
 }
 
+/* Mark glyph names for garbager. */
+void
+pdf_mark_glyph_names(const pdf_font_resource_t *pdfont, const gs_memory_t *memory)
+{
+    if (pdfont->mark_glyph == NULL) {
+	/* Synthesised bitmap fonts pass here. */
+	return;
+    }
+    if (pdfont->u.simple.Encoding != NULL) {
+	 int i;
+
+	 for (i = 0; i < 256; i++)
+	     if (pdfont->u.simple.Encoding[i].glyph != GS_NO_GLYPH)
+		pdfont->mark_glyph(memory, pdfont->u.simple.Encoding[i].glyph, pdfont->mark_glyph_data);
+     }
+    if (pdfont->FontType == ft_user_defined) {
+	const pdf_char_proc_ownership_t *pcpo = pdfont->u.simple.s.type3.char_procs;
+
+	for (; pcpo != NULL; pcpo = pcpo->font_next)
+	    pdfont->mark_glyph(memory, pcpo->glyph, pdfont->mark_glyph_data);
+    }
+}
+
 /* Put out a reference to an image as a character in a synthesized font. */
 int
 pdf_do_char_image(gx_device_pdf * pdev, const pdf_char_proc_t * pcp,
 		  const gs_matrix * pimat)
 {
-    pdf_font_resource_t *pdfont = pcp->font;
-    byte ch = pcp->char_code;
+    /* We need to choose a font, which use the charproc.
+       In most cases it is the last font, which the charproc is attached to.
+       If the charproc is substituted, it causes a font change. */
+    const pdf_char_proc_ownership_t * pcpo = pcp->owner_fonts;
+    pdf_font_resource_t *pdfont = pcpo->font;
+    byte ch = pcpo->char_code;
     pdf_text_state_values_t values;
 
     values.character_spacing = 0;
@@ -350,7 +406,7 @@ pdf_do_char_image(gx_device_pdf * pdev, const pdf_char_proc_t * pcp,
 int
 pdf_write_bitmap_fonts_Encoding(gx_device_pdf *pdev)
 {
-    const pdf_bitmap_fonts_t *pbfs = pdev->text->bitmap_fonts;
+    pdf_bitmap_fonts_t *pbfs = pdev->text->bitmap_fonts;
 
     if (pbfs->bitmap_encoding_id) {
 	stream *s;
@@ -373,6 +429,7 @@ pdf_write_bitmap_fonts_Encoding(gx_device_pdf *pdev)
 	}
 	stream_puts(s, "\n] >>\n");
 	pdf_end_separate(pdev);
+	pbfs->bitmap_encoding_id = 0;
     }
     return 0;
 }
@@ -391,11 +448,7 @@ pdf_start_charproc_accum(gx_device_pdf *pdev)
     if (code < 0)
        return code;
     pcp = (pdf_char_proc_t *)pres;
-    pcp->char_next = NULL;
-    pcp->font = NULL;
-    pcp->char_code = GS_NO_CHAR;
-    pcp->char_name.data = NULL;
-    pcp->char_name.size = 0;
+    pcp->owner_fonts = NULL;
     return 0;
 }
 
@@ -404,7 +457,7 @@ pdf_start_charproc_accum(gx_device_pdf *pdev)
  */
 int
 pdf_set_charproc_attrs(gx_device_pdf *pdev, gs_font *font, const double *pw, int narg,
-		gs_text_cache_control_t control, gs_char ch, gs_const_string *gnstr)
+		gs_text_cache_control_t control, gs_char ch)
 {
     pdf_font_resource_t *pdfont;
     pdf_resource_t *pres = pdev->accumulating_substream_resource;
@@ -415,10 +468,7 @@ pdf_set_charproc_attrs(gx_device_pdf *pdev, gs_font *font, const double *pw, int
     if (code < 0)
 	return code;
     pcp = (pdf_char_proc_t *)pres;
-    pcp->char_next = NULL;
-    pcp->font = pdfont;
-    pcp->char_code = ch;
-    pcp->char_name = *gnstr;
+    pcp->owner_fonts = NULL;
     pcp->real_width.x = pw[font->WMode && narg > 6 ? 6 : 0];
     pcp->real_width.y = pw[font->WMode && narg > 6 ? 7 : 1];
     pcp->v.x = (narg > 8 ? pw[8] : 0);
@@ -437,8 +487,8 @@ pdf_set_charproc_attrs(gx_device_pdf *pdev, gs_font *font, const double *pw, int
 	    (float)pw[3], (float)pw[4], (float)pw[5]);
 	pdfont->u.simple.s.type3.cached[ch >> 3] |= 0x80 >> (ch & 7);
     }
-    pdfont->used[ch >> 3] |= 0x80 >> (ch & 7);
     pdev->font3 = (pdf_resource_t *)pdfont;
+    pdev->substream_Resources = pdfont->u.simple.s.type3.Resources;
     return 0;
 }
 
@@ -448,7 +498,7 @@ pdf_set_charproc_attrs(gx_device_pdf *pdev, gs_font *font, const double *pw, int
 
 int
 pdf_open_aside(gx_device_pdf *pdev, pdf_resource_type_t rtype, 
-	gs_id id, pdf_resource_t **ppres, bool reserve_object_id, bool compress) 
+	gs_id id, pdf_resource_t **ppres, bool reserve_object_id, int options) 
 {
     int code;
     pdf_resource_t *pres;
@@ -468,15 +518,8 @@ pdf_open_aside(gx_device_pdf *pdev, pdf_resource_type_t rtype,
     if (s == 0)
 	return_error(gs_error_VMerror);
     pdev->strm = s;
-#if PDFW_DELAYED_STREAMS
     code = pdf_append_data_stream_filters(pdev, &writer,
-			     DATA_STREAM_NOT_BINARY | DATA_STREAM_NOLENGTH |
-			     (compress ? DATA_STREAM_COMPRESS : 0), 0);
-#else
-    code = pdf_begin_data_stream(pdev, &writer,
-			     DATA_STREAM_NOT_BINARY | DATA_STREAM_NOLENGTH |
-			     (compress ? DATA_STREAM_COMPRESS : 0), 0);
-#endif
+			     options | DATA_STREAM_NOLENGTH, pres->object->id);
     if (code < 0) {
 	pdev->strm = save_strm;
 	return code;
@@ -532,7 +575,8 @@ pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype,
 	if (pdev->sbstack[sbstack_ptr].text_state == 0)
 	    return_error(gs_error_VMerror);
     }
-    code = pdf_open_aside(pdev, rtype, id, &pres, reserve_object_id, compress);
+    code = pdf_open_aside(pdev, rtype, id, &pres, reserve_object_id, 
+		    (compress ? DATA_STREAM_COMPRESS : 0));
     if (code < 0)
 	return code;
     code = pdf_save_viewer_state(pdev, NULL);
@@ -556,8 +600,15 @@ pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype,
     pdev->sbstack[sbstack_ptr].font3 = pdev->font3;
     pdev->sbstack[sbstack_ptr].accumulating_substream_resource = pdev->accumulating_substream_resource;
     pdev->sbstack[sbstack_ptr].charproc_just_accumulated = pdev->charproc_just_accumulated;
+    pdev->sbstack[sbstack_ptr].accumulating_a_global_object = pdev->accumulating_a_global_object;
+    pdev->sbstack[sbstack_ptr].pres_soft_mask_dict = pdev->pres_soft_mask_dict;
+    pdev->sbstack[sbstack_ptr].objname = pdev->objname;
     pdev->skip_colors = false;
     pdev->charproc_just_accumulated = false;
+    pdev->pres_soft_mask_dict = NULL;
+    pdev->objname.data = NULL;
+    pdev->objname.size = 0;
+    /* Do not reset pdev->accumulating_a_global_object - it inherits. */
     pdev->sbstack_depth++;
     pdev->procsets = 0;
     pdev->font3 = 0;
@@ -608,6 +659,9 @@ pdf_exit_substream(gx_device_pdf *pdev)
     pdev->accumulating_substream_resource = pdev->sbstack[sbstack_ptr].accumulating_substream_resource;
     pdev->sbstack[sbstack_ptr].accumulating_substream_resource = 0;
     pdev->charproc_just_accumulated = pdev->sbstack[sbstack_ptr].charproc_just_accumulated;
+    pdev->accumulating_a_global_object = pdev->sbstack[sbstack_ptr].accumulating_a_global_object;
+    pdev->pres_soft_mask_dict = pdev->sbstack[sbstack_ptr].pres_soft_mask_dict;
+    pdev->objname = pdev->sbstack[sbstack_ptr].objname;
     pdev->sbstack_depth = sbstack_ptr;
     code1 = pdf_restore_viewer_state(pdev, NULL);
     if (code1 < 0 && code >= 0)
@@ -616,16 +670,8 @@ pdf_exit_substream(gx_device_pdf *pdev)
 }
 
 private bool 
-pdf_is_same_charproc1(gx_device_pdf * pdev, pdf_char_proc_t *pcp0, pdf_char_proc_t *pcp1)
+pdf_is_same_charproc_attars1(gx_device_pdf *pdev, pdf_char_proc_t *pcp0, pdf_char_proc_t *pcp1)
 {
-    if (pcp0->char_code != pcp1->char_code)
-	return false; /* We need same encoding. */
-    if (pcp0->font->u.simple.Encoding[pcp0->char_code].glyph !=
-	pcp1->font->u.simple.Encoding[pcp1->char_code].glyph)
-	return false; /* We need same encoding. */
-    if (bytes_compare(pcp0->char_name.data, pcp0->char_name.size, 
-		      pcp1->char_name.data, pcp1->char_name.size))
-	return false; /* We need same encoding. */
     if (pcp0->real_width.x != pcp1->real_width.x)
 	return false;
     if (pcp0->real_width.y != pcp1->real_width.y)
@@ -634,132 +680,147 @@ pdf_is_same_charproc1(gx_device_pdf * pdev, pdf_char_proc_t *pcp0, pdf_char_proc
 	return false;
     if (pcp0->v.y != pcp1->v.y)
 	return false;
-    if (pcp0->font->u.simple.s.type3.bitmap_font != pcp1->font->u.simple.s.type3.bitmap_font)
-	return false;
-    if (memcmp(&pcp0->font->u.simple.s.type3.FontMatrix, &pcp1->font->u.simple.s.type3.FontMatrix,
-		sizeof(pcp0->font->u.simple.s.type3.FontMatrix)))
-	return false;
-    return pdf_check_encoding_compatibility(pcp1->font, pdev->cgp->s, pdev->cgp->num_all_chars);
+    return true;
+}
+
+typedef struct charproc_compatibility_data_s {
+    const pdf_char_glyph_pairs_t *cgp;
+    pdf_font_resource_t *pdfont;
+    gs_char char_code;
+    gs_glyph glyph;
+    gs_font *font;
+} charproc_compatibility_data_t;
+
+private int 
+pdf_is_charproc_compatible(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
+{
+    charproc_compatibility_data_t *data = (charproc_compatibility_data_t *)pdev->find_resource_param;
+    pdf_char_proc_t *pcp0 = (pdf_char_proc_t *)pres0;
+    pdf_char_proc_t *pcp1 = (pdf_char_proc_t *)pres1;
+    pdf_font_resource_t *pdfont = data->pdfont;
+    pdf_char_proc_ownership_t *pcpo;
+    pdf_font_cache_elem_t **e;
+
+    /* Does it have same attributes ? */
+    if (!pdf_is_same_charproc_attars1(pdev, pcp0, pcp1))
+	return 0;
+    /* Is it from same font ? */
+    for (pcpo = pcp1->owner_fonts; pcpo != NULL; pcpo = pcpo->char_next) {
+	if (pdfont == pcpo->font)
+	    return 1;
+    }
+    /* Look for another font with same encoding,
+       because we want to reduce the number of new fonts. 
+       We restrict with ones attached to same PS font,
+       otherwise it creates too mixed fonts and disturbs word breaks.
+     */
+    for (pcpo = pcp1->owner_fonts; pcpo != NULL; pcpo = pcpo->char_next) {
+	if (pcpo->char_code != data->char_code || pcpo->glyph != data->glyph)
+	    continue; /* Need same Encoding to generate a proper ToUnicode. */
+	if (pdfont->u.simple.s.type3.bitmap_font != pcpo->font->u.simple.s.type3.bitmap_font)
+	    continue;
+	if (memcmp(&pdfont->u.simple.s.type3.FontMatrix, &pcpo->font->u.simple.s.type3.FontMatrix,
+		    sizeof(pdfont->u.simple.s.type3.FontMatrix)))
+	    continue;
+	if (data->cgp != NULL) {
+	    if (!pdf_check_encoding_compatibility(pcpo->font, data->cgp->s, data->cgp->num_all_chars))
+		continue;
+	}
+	e = pdf_locate_font_cache_elem(pdev, data->font);
+	if (e == NULL || (*e)->pdfont != pdfont)
+	    continue;
+	data->pdfont = (*e)->pdfont;
+	return 1;
+    }
+    /* Wil share with another font. */
+    return 1;
 }
 
 private int 
-pdf_is_same_charproc(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
+pdf_find_same_charproc_aux(gx_device_pdf *pdev, 
+	    pdf_font_resource_t **ppdfont, pdf_char_proc_t **ppcp)
 {
-    return pdf_is_same_charproc1(pdev, (pdf_char_proc_t *)pres0, (pdf_char_proc_t *)pres1);
-}
-
-private int 
-pdf_find_same_charproc(gx_device_pdf *pdev, 
-	    pdf_font_resource_t *pdfont, const pdf_char_glyph_pairs_t *cgp, 
-	    pdf_char_proc_t **ppcp)
-{
-    pdf_char_proc_t *pcp;
+    pdf_char_proc_ownership_t *pcpo;
     int code;
 
-    pdev->cgp = cgp;
-    for (pcp = pdfont->u.simple.s.type3.char_procs; pcp != NULL; pcp = pcp->char_next) {
-	if (*ppcp != pcp && pdf_is_same_charproc1(pdev, *ppcp, pcp)) {
-	    *ppcp = pcp;
-	    pdev->cgp = NULL;
-	    return 1;
+    /* fixme: this passes parameters to pdf_is_charproc_compatible 
+       through special gx_device_pdf field pdev->find_resource_param
+       due to prototype limitation of pdf_find_same_resource.
+       It would be better to change the client data argument type in there to void. */
+    for (pcpo = (*ppdfont)->u.simple.s.type3.char_procs; pcpo != NULL; pcpo = pcpo->char_next) {
+	pdf_char_proc_t *pcp = pcpo->char_proc;
+
+	if (*ppcp != pcp && pdf_is_same_charproc_attars1(pdev, *ppcp, pcp)) {
+    	    cos_object_t *pco0 = pcp->object;
+    	    cos_object_t *pco1 = (*ppcp)->object;
+
+	    code = pco0->cos_procs->equal(pco0, pco1, pdev);
+	    if (code < 0) {
+		return code;
+	    }
+	    if (code) {
+		*ppcp = pcp;
+		return 1;
+	    }
 	}
     }
-    pcp = *ppcp;
-    code = pdf_find_same_resource(pdev, resourceCharProc, (pdf_resource_t **)ppcp, pdf_is_same_charproc);
-    pdev->cgp = NULL;
-    if (code <= 0)
-	return code;
-    /* fixme : do we need more checks here ? */
-    return 1;
+    return pdf_find_same_resource(pdev, resourceCharProc, (pdf_resource_t **)ppcp, pdf_is_charproc_compatible);
+}
+private int 
+pdf_find_same_charproc(gx_device_pdf *pdev, 
+	    pdf_font_resource_t **ppdfont, const pdf_char_glyph_pairs_t *cgp, 
+	    pdf_char_proc_t **ppcp, gs_glyph glyph, gs_char char_code,
+	    gs_font *font)
+{
+    charproc_compatibility_data_t data;
+    int code;
+
+    data.cgp = cgp;
+    data.pdfont = *ppdfont;
+    data.char_code = char_code;
+    data.glyph = glyph;
+    data.font = font;
+    pdev->find_resource_param = &data;
+    code = pdf_find_same_charproc_aux(pdev, ppdfont, ppcp);
+    pdev->find_resource_param = NULL;
+    *ppdfont = data.pdfont;
+    return code;
 }
 
 private bool
 pdf_is_charproc_defined(gx_device_pdf *pdev, pdf_font_resource_t *pdfont, gs_char ch)
 {
-    pdf_char_proc_t *pcp;
+    pdf_char_proc_ownership_t *pcpo;
 
-    for (pcp = pdfont->u.simple.s.type3.char_procs; pcp != NULL; pcp = pcp->char_next) {
-	if (pcp->char_code == ch) {
+    for (pcpo = pdfont->u.simple.s.type3.char_procs; pcpo != NULL; pcpo = pcpo->char_next) {
+	if (pcpo->char_code == ch)
 	    return true;
-	}
     }
     return false;
 }
 
-/*
- * Complete charproc accumulation for a Type 3 font.
- */
-int
-pdf_end_charproc_accum(gx_device_pdf *pdev, gs_font *font, const pdf_char_glyph_pairs_t *cgp) 
-{
-    int code;
-    pdf_resource_t *pres = (pdf_resource_t *)pdev->accumulating_substream_resource;
-    /* We could use pdfont->u.simple.s.type3.char_procs insted the thing above
-       unless the font is defined recursively.
-       But we don't want such assumption. */
-    pdf_char_proc_t *pcp = (pdf_char_proc_t *)pres;
+private int
+complete_adding_char(gx_device_pdf *pdev, gs_font *font, 
+		     gs_glyph glyph, gs_char ch, pdf_char_proc_t *pcp,
+		     const gs_const_string *gnstr)
+{   
     pdf_font_resource_t *pdfont;
-    gs_char ch = pcp->char_code;
     double *real_widths;
     byte *glyph_usage;
     int char_cache_size, width_cache_size;
-    gs_glyph glyph0;
-    bool checking_glyph_variation = false;
-    int i;
+    pdf_encoding_element_t *pet;
+    int code;
 
-    code = pdf_attached_font_resource(pdev, font, &pdfont, NULL, NULL, NULL, NULL);
-    if (code < 0)
-	return code;
-    if (pdfont != (pdf_font_resource_t *)pdev->font3)
-	return_error(gs_error_unregistered); /* Must not happen. */
-    if (ch == GS_NO_CHAR)
-	return_error(gs_error_unregistered); /* Must not happen. */
-    if (ch >= 256)
-	return_error(gs_error_unregistered); /* Must not happen. */
-    code = pdf_exit_substream(pdev);
-    if (code < 0)
-	return code;
-    if (pdfont->used[ch >> 3] & (0x80 >> (ch & 7))) {
-	if (!(pdfont->u.simple.s.type3.cached[ch >> 3] & (0x80 >> (ch & 7)))) {
-	    checking_glyph_variation = true;
-	    code = pdf_find_same_charproc(pdev, pdfont, cgp, &pcp);
-	    if (code < 0)
-		return code;
-	    if (code != 0) {
-		code = pdf_cancel_resource(pdev, pres, resourceCharProc);
-		if (code < 0)
-		    return code;
-		pdf_forget_resource(pdev, pres, resourceCharProc);
-		if (pcp->font != pdfont) {
-		    code = pdf_attach_font_resource(pdev, font, pcp->font);
-		    if (code < 0)
-			return code;
-		}
-		pdev->charproc_just_accumulated = true;
-		return 0;
-	    }
-	    if (pdf_is_charproc_defined(pdev, pdfont, ch)) {
-		code = pdf_make_font3_resource(pdev, font, &pdfont);
-		if (code < 0)
-		    return code;
-		code = pdf_attach_font_resource(pdev, font, pdfont);
-		if (code < 0)
-		    return code;
-	    }
-	}
-    } 
-    pdf_reserve_object_id(pdev, pres, 0);
     code = pdf_attached_font_resource(pdev, font, &pdfont,
 		&glyph_usage, &real_widths, &char_cache_size, &width_cache_size);
+    if (code < 0)
+	return code;
+    code = pdf_attach_charproc(pdev, pdfont, pcp, glyph, ch, gnstr);
     if (code < 0)
 	return code;
     if (ch >= char_cache_size || ch >= width_cache_size)
 	return_error(gs_error_unregistered); /* Must not happen. */
-    if (checking_glyph_variation)
-	pdev->charproc_just_accumulated = true;
-    glyph0 = font->procs.encode_char(font, ch, GLYPH_SPACE_NAME);
-    pcp->char_next = pdfont->u.simple.s.type3.char_procs;
-    pdfont->u.simple.s.type3.char_procs = pcp;
+    pet = &pdfont->u.simple.Encoding[ch];
     pdfont->Widths[ch] = pcp->real_width.x;
     real_widths[ch * 2    ] = pcp->real_width.x;
     real_widths[ch * 2 + 1] = pcp->real_width.y;
@@ -769,23 +830,118 @@ pdf_end_charproc_accum(gx_device_pdf *pdev, gs_font *font, const pdf_char_glyph_
 	pdfont->u.simple.v[ch].x = pcp->v.x;
 	pdfont->u.simple.v[ch].y = pcp->v.x;
     }
-    for (i = 0; i < 256; i++) {
-	gs_glyph glyph = font->procs.encode_char(font, i, 
-		    font->FontType == ft_user_defined ? GLYPH_SPACE_NOGEN
-						      : GLYPH_SPACE_NAME);
+    pet->glyph = glyph;
+    pet->str = *gnstr;
+    pet->is_difference = true;
+    if (pdfont->u.simple.LastChar < (int)ch)
+	pdfont->u.simple.LastChar = (int)ch;
+    if (pdfont->u.simple.FirstChar > (int)ch)
+	pdfont->u.simple.FirstChar = (int)ch;
+    return 0;
+}
 
-	if (glyph == glyph0) {
-	    real_widths[i * 2    ] = real_widths[ch * 2    ];
-	    real_widths[i * 2 + 1] = real_widths[ch * 2 + 1];
-	    glyph_usage[i / 8] |= 0x80 >> (i & 7);
-	    pdfont->used[i >> 3] |= 0x80 >> (i & 7);
-	    pdfont->u.simple.v[i] = pdfont->u.simple.v[ch];
-	    pdfont->Widths[i] = pdfont->Widths[ch];
-	}
+private int
+pdf_char_widths_from_charprocs(gx_device_pdf *pdev, gs_font *font)
+{
+    pdf_font_resource_t *pdfont;
+    double *real_widths;
+    byte *glyph_usage;
+    int char_cache_size, width_cache_size;
+    pdf_char_proc_ownership_t *pcpo;
+    int code;
+
+    code = pdf_attached_font_resource(pdev, font, &pdfont,
+		&glyph_usage, &real_widths, &char_cache_size, &width_cache_size);
+    if (code < 0)
+	return code;
+    for (pcpo = pdfont->u.simple.s.type3.char_procs; pcpo != NULL; pcpo = pcpo->char_next) {
+	pdf_char_proc_t *pcp = pcpo->char_proc;
+	gs_char ch = pcpo->char_code;
+
+	real_widths[ch * 2    ] = pcp->real_width.x;
+	real_widths[ch * 2 + 1] = pcp->real_width.y;
+	glyph_usage[ch / 8] |= 0x80 >> (ch & 7);
     }
     return 0;
 }
 
+
+/*
+ * Complete charproc accumulation for a Type 3 font.
+ */
+int
+pdf_end_charproc_accum(gx_device_pdf *pdev, gs_font *font, const pdf_char_glyph_pairs_t *cgp, 
+		       gs_glyph glyph, gs_char output_char_code, const gs_const_string *gnstr) 
+{
+    int code;
+    pdf_resource_t *pres = (pdf_resource_t *)pdev->accumulating_substream_resource;
+    /* We could use pdfont->u.simple.s.type3.char_procs insted the thing above
+       unless the font is defined recursively.
+       But we don't want such assumption. */
+    pdf_char_proc_t *pcp = (pdf_char_proc_t *)pres;
+    pdf_font_resource_t *pdfont;
+    gs_char ch = output_char_code;
+    bool checking_glyph_variation = false;
+
+    if (ch == GS_NO_CHAR)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (ch >= 256)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    code = pdf_attached_font_resource(pdev, font, &pdfont, NULL, NULL, NULL, NULL);
+    if (code < 0)
+	return code;
+    if (pdfont != (pdf_font_resource_t *)pdev->font3)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    code = pdf_exit_substream(pdev);
+    if (code < 0)
+	return code;
+    if (!(pdfont->used[ch >> 3] & (0x80 >> (ch & 7))) ||
+	!(pdfont->u.simple.s.type3.cached[ch >> 3] & (0x80 >> (ch & 7)))) {
+	/* First appearence or not cached - check for duplicates. */
+	pdf_font_resource_t *pdfont1 = pdfont;
+
+	checking_glyph_variation = true;
+	/* CAUTION : a possible font change. */
+	code = pdf_find_same_charproc(pdev, &pdfont, cgp, &pcp, glyph, ch, font);
+	if (code < 0)
+	    return code;
+	if (code != 0) {
+	    code = pdf_cancel_resource(pdev, pres, resourceCharProc);
+	    if (code < 0)
+		return code;
+	    pdf_forget_resource(pdev, pres, resourceCharProc);
+	    if (pdfont1 != pdfont) {
+		code = pdf_attach_font_resource(pdev, font, pdfont);
+		if (code < 0)
+		    return code;
+		code = pdf_char_widths_from_charprocs(pdev, font);
+		if (code < 0)
+		    return code;
+	    }
+	    pdev->charproc_just_accumulated = true;
+	    return complete_adding_char(pdev, font, glyph, ch, pcp, gnstr);
+	}
+	if (pdf_is_charproc_defined(pdev, pdfont, ch)) {
+	    /* Encoding conflict after a font change. */
+	    gs_font *base_font = font, *below;
+
+	    while ((below = base_font->base) != base_font &&
+		    base_font->procs.same_font(base_font, below, FONT_SAME_OUTLINES))
+		base_font = below;
+	    code = pdf_make_font3_resource(pdev, base_font, &pdfont);
+	    if (code < 0)
+		return code;
+	    code = pdf_attach_font_resource(pdev, font, pdfont);
+	    if (code < 0)
+		return code;
+	}
+    } 
+    pdf_reserve_object_id(pdev, pres, 0);
+    if (checking_glyph_variation)
+	pdev->charproc_just_accumulated = true;
+    return complete_adding_char(pdev, font, glyph, ch, pcp, gnstr);
+}
+
 /* Add procsets to substream Resources. */
 int
 pdf_add_procsets(cos_dict_t *pcd, pdf_procset_t procsets)
@@ -817,6 +973,12 @@ pdf_add_resource(gx_device_pdf *pdev, cos_dict_t *pcd, const char *key, pdf_reso
 	int code;
 	char buf[10 + (sizeof(long) * 8 / 3 + 1)], buf1[sizeof(pres->rname) + 1];
 
+	if (pdev->ForOPDFRead && !pres->global && pdev->accumulating_a_global_object) {
+	    pres->global = true;
+	    code = cos_dict_put_c_key_bool((cos_dict_t *)pres->object, "/.Global", true);
+	    if (code < 0)
+		return code;
+	}
 	sprintf(buf, "%ld 0 R\n", pres->object->id);
 	if (v != NULL) {
 	    if (v->value_type != COS_VALUE_OBJECT && 
