@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsfcid.c 5515 2004-11-19 04:39:11Z ray $ */
+/* $Id: gsfcid.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Support for CID-keyed fonts */
 #include "memory_.h"
 #include "gx.h"
@@ -177,9 +174,8 @@ gs_font_cid0_enumerate_glyph(gs_font *font, int *pindex,
     *pindex = 0;
     return 0;
 }
-/* Get the FontMatrix for the current type0 font 	*/
-/* Assumes that the glyph has already been accessed and	*/
-/* the index is valid.					*/
+
+/* Return the font from the FDArray at the given index */
 const gs_font *
 gs_cid0_indexed_font(const gs_font *font, int fidx)
 {
@@ -191,3 +187,20 @@ gs_cid0_indexed_font(const gs_font *font, int fidx)
     }
     return (const gs_font*) (pfont->cidata.FDArray[fidx]);
 }
+
+/* Check whether a CID font has a Type 2 subfont. */
+bool
+gs_cid0_has_type2(const gs_font *font)
+{
+    gs_font_cid0 *const pfont = (gs_font_cid0 *)font;
+    int i;
+
+    if (font->FontType != ft_CID_encrypted) {
+	eprintf1("Unexpected font type: %d\n", font->FontType);
+        return false;
+    }
+    for (i = 0; i < pfont->cidata.FDArray_size; i++)
+	if (((const gs_font *)pfont->cidata.FDArray[i])->FontType == ft_encrypted2)
+	    return true;
+    return false;
+}
