@@ -1,22 +1,18 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsfcmap.c 5567 2004-12-08 21:35:13Z stefan $ */
+/* $Id: gsfcmap.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* CMap character decoding */
-#include <assert.h>
 #include "memory_.h"
 #include "string_.h"
 #include "gx.h"
@@ -444,8 +440,7 @@ gs_cmap_ToUnicode_decode_next(const gs_cmap_t *pcmap, const gs_const_string *str
 		     uint *pindex, uint *pfidx,
 		     gs_char *pchr, gs_glyph *pglyph)
 {
-    assert(0); /* Unsupported, because never used. */
-    return 0;
+    return_error(gs_error_unregistered);
 }
 
 private void
