@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsht.c 5215 2004-08-04 19:36:13Z stefan $ */
+/*$Id: gsht.c 6671 2006-03-22 07:05:00Z raph $ */
 /* setscreen operator for Ghostscript library */
 #include "memory_.h"
 #include "string_.h"
@@ -341,6 +338,7 @@ gx_ht_copy_ht_order(gx_ht_order * pdest, gx_ht_order * psrc, gs_memory_t * mem)
         memcpy(pdest->bit_data, psrc->bit_data,
 		psrc->num_bits * psrc->procs->bit_data_elt_size);
     pdest->wse = psrc->wse;
+    pdest->wts = psrc->wts;
     pdest->transfer = psrc->transfer;
     rc_increment(pdest->transfer);
     return 0;
@@ -693,6 +691,15 @@ gs_color_name_component_number(gx_device * dev, const char * pname,
 	    num_colorant = check_colorant_name(dev, "Yellow");
 	else if (check_name("Gray", pname, name_size))
 	    num_colorant = check_colorant_name(dev, "Black");
+	/*
+	 * The device will return GX_DEVICE_COLOR_MAX_COMPONENTS if the
+	 * colorant is logically present in the device but not being used
+	 * because a SeparationOrder parameter is specified.  Since we are
+	 * using this value to indicate 'Default', we use -1 to indicate
+	 * that the colorant is not really being used.
+	 */
+	if (num_colorant == GX_DEVICE_COLOR_MAX_COMPONENTS)
+	    num_colorant = -1;
 
 #undef check_colorant_name
 #undef check_colorant_name_length
@@ -766,7 +773,13 @@ gs_cname_to_colorant_number(gs_state * pgs, byte * pname, uint name_size,
  *  num_comp    For the operand halftone, this is the number of halftone
  *              components included in the specification. For the device
  *              halftone in the imager state, this is always the same as
- *              the number of color model components.
+ *              the number of color model components (see num_dev_comp).
+ *
+ *  num_dev_comp The number of components in the device process color model
+ *		when the operand halftone was created.  With some compositor
+ *		devices (for example PDF 1.4) we can have differences in the
+ *		process color model of the compositor versus the output device.
+ *		These compositor devices do not halftone.
  *
  *  components  For the operand halftone, this field is non-null only if
  *              multiple halftones are provided. In that case, the size
@@ -951,7 +964,7 @@ gx_imager_dev_ht_install(
     const gx_device *       dev )
 {
     gx_device_halftone      dht;
-    int                     num_comps = dev->color_info.num_components;
+    int                     num_comps = pdht->num_dev_comp;
     int                     i, code = 0;
     bool                    used_default = false;
     int                     lcm_width = 1, lcm_height = 1;
@@ -972,7 +985,7 @@ gx_imager_dev_ht_install(
                           "gx_imager_dev_ht_install(components)" );
     if (dht.components == NULL)
 	return_error(gs_error_VMerror);
-    dht.num_comp = num_comps;
+    dht.num_comp = dht.num_dev_comp = num_comps;
     /* lcm_width, lcm_height are filled in later */
 
     /* initialize the components array */
@@ -1081,7 +1094,7 @@ gx_imager_dev_ht_install(
                 code = gs_error_VMerror;
             else
                 porder->wts = wts;
-        } else {
+        } else if (porder->wts == 0) {
             uint   w = porder->width, h = porder->full_height;
             int    dw = igcd(lcm_width, w), dh = igcd(lcm_height, h);
 
@@ -1223,6 +1236,7 @@ gx_ht_install(gs_state * pgs, const gs_halftone * pht,
     gs_halftone *new_ht;
     int code;
 
+    pdht->num_dev_comp = pgs->device->color_info.num_components;
     if (old_ht != 0 && old_ht->rc.memory == mem &&
 	old_ht->rc.ref_count == 1
 	)
