@@ -1,20 +1,17 @@
-/* Copyright (C) 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsistate.c 5215 2004-08-04 19:36:13Z stefan $ */
+/* $Id: gsistate.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Imager state housekeeping */
 #include "gx.h"
 #include "gserrors.h"
@@ -28,6 +25,7 @@
 #include "gxistate.h"
 #include "gzht.h"
 #include "gzline.h"
+#include "gxfmap.h"
 
 /******************************************************************************
  * See gsstate.c for a discussion of graphics/imager state memory management. *
@@ -92,12 +90,6 @@ private RELOC_PTRS_BEGIN(imager_state_reloc_ptrs)
 
 /* Initialize an imager state, other than the parts covered by */
 /* gs_imager_state_initial. */
-private float
-imager_null_transfer(floatp gray, const gx_transfer_map * pmap)
-{
-    return gray;
-}
-
 int
 gs_imager_state_initialize(gs_imager_state * pis, gs_memory_t * mem)
 {
@@ -124,7 +116,7 @@ gs_imager_state_initialize(gs_imager_state * pis, gs_memory_t * mem)
 		      gx_transfer_map, &st_transfer_map,
 		      mem, return_error(gs_error_VMerror),
 		      "gs_imager_state_init(transfer)", 1);
-    pis->set_transfer.gray->proc = imager_null_transfer;
+    pis->set_transfer.gray->proc = gs_identity_transfer;
     pis->set_transfer.gray->id = gs_next_ids(pis->memory, 1);
     pis->set_transfer.gray->values[0] = frac_0;
     pis->set_transfer.red =
