@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gspaint.c 4887 2004-03-13 18:28:52Z igor $ */
+/* $Id: gspaint.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Painting procedures for Ghostscript library */
 #include "math_.h"		/* for fabs */
 #include "gx.h"
@@ -237,17 +234,19 @@ alpha_buffer_init(gs_state * pgs, fixed extra_x, fixed extra_y, int alpha_bits)
 }
 
 /* Release an alpha buffer. */
-private void
+private int
 alpha_buffer_release(gs_state * pgs, bool newpath)
 {
     gx_device_memory *mdev =
-    (gx_device_memory *) gs_currentdevice_inline(pgs);
+	(gx_device_memory *) gs_currentdevice_inline(pgs);
+    int code = (*dev_proc(mdev, close_device)) ((gx_device *) mdev);
 
-    (*dev_proc(mdev, close_device)) ((gx_device *) mdev);
-    scale_paths(pgs, -mdev->log2_scale.x, -mdev->log2_scale.y,
+    if (code >= 0)
+	scale_paths(pgs, -mdev->log2_scale.x, -mdev->log2_scale.y,
 		!(newpath && !gx_path_is_shared(pgs->path)));
     /* Reference counting will free mdev. */
     gx_set_device_only(pgs, mdev->target);
+    return code;
 }
 
 /* Fill the current path using a specified rule. */
@@ -261,8 +260,12 @@ fill_with_rule(gs_state * pgs, int rule)
     if (pgs->in_charpath)
 	code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
 				     pgs->in_charpath);
-    else {
-	int abits, acode;
+    else if (gs_is_null_device(pgs->device)) {
+	/* Handle separately to prevent gs_state_color_load - bug 688308. */
+	gs_newpath(pgs);
+	code = 0;
+    } else {
+	int abits, acode, rcode = 0;
 
 	gx_set_dev_color(pgs);
 	code = gs_state_color_load(pgs);
@@ -279,9 +282,11 @@ fill_with_rule(gs_state * pgs, int rule)
 	code = gx_fill_path(pgs->path, pgs->dev_color, pgs, rule,
 			    pgs->fill_adjust.x, pgs->fill_adjust.y);
 	if (acode > 0)
-	    alpha_buffer_release(pgs, code >= 0);
+	    rcode = alpha_buffer_release(pgs, code >= 0);
 	if (code >= 0)
 	    gs_newpath(pgs);
+	if (code >= 0 && rcode < 0)
+	    code = rcode;
 
     }
     return code;
@@ -321,8 +326,12 @@ gs_stroke(gs_state * pgs)
 	}
 	code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
 				     pgs->in_charpath);
+    } else if (gs_is_null_device(pgs->device)) {
+	/* Handle separately to prevent gs_state_color_load. */
+	gs_newpath(pgs);
+	code = 0;
     } else {
-	int abits, acode;
+	int abits, acode, rcode = 0;
 
 	gx_set_dev_color(pgs);
 	code = gs_state_color_load(pgs);
@@ -373,11 +382,13 @@ gs_stroke(gs_state * pgs)
 	    gs_setflat(pgs, orig_flatness);
 	    gx_path_free(&spath, "gs_stroke");
 	    if (acode > 0)
-		alpha_buffer_release(pgs, code >= 0);
+		rcode = alpha_buffer_release(pgs, code >= 0);
 	} else
 	    code = gx_stroke_fill(pgs->path, pgs);
 	if (code >= 0)
 	    gs_newpath(pgs);
+	if (code >= 0 && rcode < 0)
+	    code = rcode;
     }
     return code;
 }
