@@ -1,20 +1,17 @@
-/* Copyright (C) 1990, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gstype1.c 5465 2004-10-28 08:39:21Z igor $ */
+/* $Id: gstype1.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Adobe Type 1 charstring interpreter */
 #include "math_.h"
 #include "memory_.h"
@@ -250,9 +247,6 @@ gs_type1_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
                 code = t1_hinter__rcurveto(h, cs0, cs1, cs2, cs3, cs4, cs5);
 		goto cc;
 	    case cx_endchar:
-                code = t1_hinter__endchar(h, (pcis->seac_accent >= 0));
-		if (code < 0)
-		    return code;
                 if (pcis->seac_accent < 0) {
                     code = t1_hinter__endglyph(h);
 		    if (code < 0)
@@ -260,7 +254,8 @@ gs_type1_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
 		    code = gx_setcurrentpoint_from_path(pcis->pis, pcis->path);
 		    if (code < 0)
 			return code;
-		}
+		} else
+		    pcis->seac_flag = true;
 		code = gs_type1_endchar(pcis);
 		if (code == 1) {
 		    /* do accent of seac */
@@ -288,16 +283,18 @@ gs_type1_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
                 code = t1_hinter__closepath(h);
 		goto cc;
 	    case c1_hsbw:
-                if (!h->seac_flag) {
+                if (!pcis->seac_flag) {
 		    fixed sbx = cs0, sby = fixed_0, wx = cs1, wy = fixed_0;
 
-		    if (pcis->sb_set) {
-			sbx = pcis->lsb.x;
-			sby = pcis->lsb.y;
-		    }
-		    if (pcis->width_set) {
-			wx = pcis->width.x;
-			wy = pcis->width.y;
+		    if (pcis->seac_accent < 0) {
+			if (pcis->sb_set) {
+			    sbx = pcis->lsb.x;
+			    sby = pcis->lsb.y;
+			}
+			if (pcis->width_set) {
+			    wx = pcis->width.x;
+			    wy = pcis->width.y;
+			}
 		    }
 		    code = t1_hinter__sbw(h, sbx, sby, wx, wy);
                 } else
@@ -382,7 +379,7 @@ rsbw:		/* Give the caller the opportunity to intervene. */
 			cip = ipsp->cs_data.bits.data;
 			goto call;
 		    case ce1_sbw:
-                        if (!h->seac_flag)
+                        if (!pcis->seac_flag)
                             code = t1_hinter__sbw(h, cs0, cs1, cs2, cs3);
                         else
                             code = t1_hinter__sbw_seac(h, cs0 + pcis->adxy.x , cs1 + pcis->adxy.y);
@@ -513,9 +510,9 @@ rsbw:		/* Give the caller the opportunity to intervene. */
 			    return_error(code);
 			goto pushed;
 		    case ce1_setcurrentpoint:
-			t1_hinter__setcurrentpoint(h, cs0, cs1);
 			cs0 += pcis->adxy.x;
 			cs1 += pcis->adxy.y;
+			t1_hinter__setcurrentpoint(h, cs0, cs1);
 			cnext;
 		    default:
 			return_error(gs_error_invalidfont);
