@@ -1,20 +1,17 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclpath.c 5408 2004-10-07 05:18:34Z ray $ */
+/*$Id: gxclpath.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Higher-level path operations for band lists */
 #include "math_.h"
 #include "memory_.h"
@@ -289,6 +286,30 @@ cmd_check_fill_known(gx_device_clist_writer *cdev, const gs_imager_state *pis,
 	*punknown |= clip_path_known;
 }
 
+/* Compute the written CTM length. */
+int
+cmd_write_ctm_return_length(gx_device_clist_writer * cldev, const gs_matrix *m)
+{
+    stream s;
+
+    s_init(&s, cldev->memory);
+    swrite_position_only(&s);
+    sput_matrix(&s, m);
+    return (uint)stell(&s);
+}
+
+/* Write out CTM. */
+int
+cmd_write_ctm(const gs_matrix *m, byte *dp, int len)
+{
+    stream s;
+
+    s_init(&s, NULL);
+    swrite_string(&s, dp + 1, len);
+    sput_matrix(&s, m);
+    return 0;
+}
+
 /* Write out values of any unknown parameters. */
 int
 cmd_write_unknown(gx_device_clist_writer * cldev, gx_clist_state * pcls,
@@ -375,18 +396,14 @@ cmd_write_unknown(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	pcls->known |= fill_adjust_known;
     }
     if (unknown & ctm_known) {
-	stream s;
-	uint len;
+	int len = cmd_write_ctm_return_length(cldev, &ctm_only(&cldev->imager_state));
 
-	s_init(&s, cldev->memory);
-	swrite_position_only(&s);
-	sput_matrix(&s, (const gs_matrix *)&cldev->imager_state.ctm);
-	len = (uint)stell(&s);
 	code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_set_ctm, len + 1);
 	if (code < 0)
 	    return code;
-	swrite_string(&s, dp + 1, len);
-	sput_matrix(&s, (const gs_matrix *)&cldev->imager_state.ctm);
+	code = cmd_write_ctm(&ctm_only(&cldev->imager_state), dp, len);
+	if (code < 0)
+	    return code;
 	pcls->known |= ctm_known;
     }
     if (unknown & dash_known) {
