@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxpcopy.c 5355 2004-09-20 13:06:10Z igor $ */
+/* $Id: gxpcopy.c 6698 2006-04-02 00:17:36Z ray $ */
 /* Path copying and flattening */
 #include "math_.h"
 #include "gx.h"
@@ -404,7 +401,7 @@ gx_curve_monotonize(gx_path * ppath, const curve_segment * pc)
 	double omt = 1 - ti, omt2 = omt * omt, omt3 = omt2 * omt;
 	double x = x0 * omt3 + 3 * pc->p1.x * omt2 * ti + 3 * pc->p2.x * omt * t2 + pc->pt.x * t3;
 	double y = y0 * omt3 + 3 * pc->p1.y * omt2 * ti + 3 * pc->p2.y * omt * t2 + pc->pt.y * t3;
-	double ddx = (c[i] & 1 ? 0 : ax * t2 + bx * ti + cx); /* Suppress noize. */
+	double ddx = (c[i] & 1 ? 0 : ax * t2 + bx * ti + cx); /* Suppress noise. */
 	double ddy = (c[i] & 2 ? 0 : ay * t2 + by * ti + cy);
 	fixed dx = (fixed)(ddx + 0.5);
 	fixed dy = (fixed)(ddy + 0.5);
@@ -415,7 +412,7 @@ gx_curve_monotonize(gx_path * ppath, const curve_segment * pc)
 	ry = (fixed)(dy * (t[i] - tp) / 3 + 0.5);
 	sx = (fixed)(x + 0.5);
 	sy = (fixed)(y + 0.5);
-	/* Suppress the derivative sign noize near a beak : */
+	/* Suppress the derivative sign noise near a peak : */
 	if ((double)(sx - px) * qx + (double)(sy - py) * qy < 0)
 	    qx = -qx, qy = -qy;
 	if ((double)(sx - px) * rx + (double)(sy - py) * ry < 0)
@@ -435,7 +432,7 @@ gx_curve_monotonize(gx_path * ppath, const curve_segment * pc)
     sy = pc->pt.y;
     rx = (fixed)((pc->pt.x - pc->p2.x) * tt + 0.5);
     ry = (fixed)((pc->pt.y - pc->p2.y) * tt + 0.5);
-    /* Suppress the derivative sign noize near peaks : */
+    /* Suppress the derivative sign noise near peaks : */
     if ((double)(sx - px) * qx + (double)(sy - py) * qy < 0)
 	qx = -qx, qy = -qy;
     if ((double)(sx - px) * rx + (double)(sy - py) * ry < 0)
@@ -587,3 +584,122 @@ gx_curve_monotonic_points(fixed v0, fixed v1, fixed v2, fixed v3,
     }
 }
 
+/* ---------------- Path optimization for the filling algorithm. ---------------- */
+
+private bool
+find_contacting_segments(const subpath *sp0, segment *sp0last, 
+			 const subpath *sp1, segment *sp1last, 
+			 segment **sc0, segment **sc1)
+{
+    segment *s0, *s1;
+    const segment *s0s, *s1s;
+    int count0, count1, search_limit = 50;
+    int min_length = fixed_1 * 1;
+
+    /* This is a simplified algorithm, which only checks for quazi-colinear vertical lines.
+       "Quazi-vertical" means dx <= 1 && dy >= min_length . */
+    /* To avoid a big unuseful expence of the processor time,
+       we search the first subpath from the end
+       (assuming that it was recently merged near the end), 
+       and restrict the search with search_limit segments 
+       against a quadratic scanning of two long subpaths. 
+       Thus algorithm is not necessary finds anything contacting.
+       Instead it either quickly finds something, or maybe not. */
+    for (s0 = sp0last, count0 = 0; count0 < search_limit && s0 != (segment *)sp0; s0 = s0->prev, count0++) {
+	s0s = s0->prev;
+	if (s0->type == s_line && (s0s->pt.x == s0->pt.x ||
+	    (any_abs(s0s->pt.x - s0->pt.x) == 1 && any_abs(s0s->pt.y - s0->pt.y) > min_length))) {
+	    for (s1 = sp1last, count1 = 0; count1 < search_limit && s1 != (segment *)sp1; s1 = s1->prev, count1++) {
+		s1s = s1->prev;
+		if (s1->type == s_line && (s1s->pt.x == s1->pt.x || 
+		    (any_abs(s1s->pt.x - s1->pt.x) == 1 && any_abs(s1s->pt.y - s1->pt.y) > min_length))) {
+		    if (s0s->pt.x == s1s->pt.x || s0->pt.x == s1->pt.x || s0->pt.x == s1s->pt.x || s0s->pt.x == s1->pt.x) {
+			if (s0s->pt.y < s0->pt.y && s1s->pt.y > s1->pt.y) {
+			    fixed y0 = max(s0s->pt.y, s1->pt.y);
+			    fixed y1 = min(s0->pt.y, s1s->pt.y);
+
+			    if (y0 <= y1) {
+				*sc0 = s0;
+				*sc1 = s1;
+				return true;
+			    }
+			}
+			if (s0s->pt.y > s0->pt.y && s1s->pt.y < s1->pt.y) {
+			    fixed y0 = max(s0->pt.y, s1s->pt.y);
+			    fixed y1 = min(s0s->pt.y, s1->pt.y);
+
+			    if (y0 <= y1) {
+				*sc0 = s0;
+				*sc1 = s1;
+				return true;
+			    }
+			}
+		    }
+		}
+	    }
+	}
+    }
+    return false;
+}
+
+int
+gx_path_merge_contacting_contours(gx_path *ppath)
+{
+    /* Now this is a simplified algorithm,
+       which merge only contours by a common quazi-vertical line. */
+    int window = 5/* max spot holes */ * 6/* segments per subpath */;
+    subpath *sp0 = ppath->segments->contents.subpath_first;
+
+    for (; sp0 != NULL; sp0 = (subpath *)sp0->last->next) {
+	segment *sp0last = sp0->last;
+	subpath *sp1 = (subpath *)sp0last->next, *spnext;
+	subpath *sp1p = sp0;
+	int count;
+	
+	for (count = 0; sp1 != NULL && count < window; sp1 = spnext, count++) {
+	    segment *sp1last = sp1->last;
+	    segment *sc0, *sc1;
+		
+	    spnext = (subpath *)sp1last->next;
+	    if (find_contacting_segments(sp0, sp0last, sp1, sp1last, &sc0, &sc1)) {
+		/* Detach the subpath 1 from the path: */
+		sp1->prev->next = sp1last->next;
+		if (sp1last->next != NULL)
+		    sp1last->next->prev = sp1->prev;
+		sp1->prev = 0;
+		sp1last->next = 0;
+		/* Change 'closepath' of the subpath 1 to a line (maybe degenerate) : */
+		if (sp1last->type == s_line_close)
+		    sp1last->type = s_line;
+		/* Rotate the subpath 1 to sc1 : */
+		{   segment *old_first = sp1->next;
+
+		    /* Detach s_start and make a loop : */
+		    sp1last->next = old_first;
+		    old_first->prev = sp1last;
+		    /* Unlink before sc1 : */
+		    sp1last = sc1->prev;
+		    sc1->prev->next = 0;
+		    sc1->prev = 0; /* Safety. */
+		    /* sp1 is not longer in use. Free it : */
+		    if (ppath->segments->contents.subpath_current == sp1) {
+			ppath->segments->contents.subpath_current = sp1p;
+		    }
+		    gs_free_object(gs_memory_stable(ppath->memory), sp1, "gx_path_merge_contacting_contours");
+		    sp1 = 0; /* Safety. */
+		}
+		/* Insert the subpath 1 into the subpath 0 before sc0 :*/
+		sc0->prev->next = sc1;
+		sc1->prev = sc0->prev;
+		sp1last->next = sc0;
+		sc0->prev = sp1last;
+		/* Remove degenearte "bridge" segments : (fixme: Not done due to low importance). */
+		/* Edit the subpath count : */
+		ppath->subpath_count--;
+	    } else
+		sp1p = sp1;
+	}
+    }
+    return 0;
+}
+
