@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1996, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: idict.c 5250 2004-08-19 19:33:09Z stefan $ */
+/* $Id: idict.c 6772 2006-05-17 15:32:18Z giles $ */
 /* Dictionary implementation */
 #include "math_.h"		/* for frexp */
 #include "string_.h"		/* for strlen */
@@ -721,6 +718,7 @@ dict_resize(ref * pdref, uint new_size, dict_stack_t *pds)
     dict *pdict = pdref->value.pdict;
     gs_ref_memory_t *mem = dict_memory(pdict);
     uint new_mask = imemory_new_mask(mem);
+    ushort orig_attrs = r_type_attrs(&pdict->values) & (a_all | a_executable);
     dict dnew;
     ref drto;
     int code;
@@ -770,6 +768,7 @@ dict_resize(ref * pdref, uint new_size, dict_stack_t *pds)
 	gs_free_ref_array(mem, &pdict->keys, "dict_resize(old keys)");
     ref_assign(&pdict->keys, &dnew.keys);
     ref_assign(&pdict->values, &dnew.values);
+    r_store_attrs(&pdict->values, a_all | a_executable, orig_attrs);
     ref_save_in(dict_memory(pdict), pdref, &pdict->maxlength,
 		"dict_resize(maxlength)");
     d_set_maxlength(pdict, new_size);
