@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: igc.c 5390 2004-09-28 05:34:29Z giles $ */
+/* $Id: igc.c 6706 2006-04-14 19:49:13Z leonardo $ */
 /* Garbage collector for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
@@ -96,7 +93,7 @@ refs_proc_reloc(igc_reloc_refs);	/* in igcref.c */
 /* Define this GC's procedure vector. */
 private const gc_procs_with_refs_t igc_procs = {
     igc_reloc_struct_ptr, igc_reloc_string, igc_reloc_const_string,
-    igc_reloc_ref_ptr, igc_reloc_refs
+    igc_reloc_param_string, igc_reloc_ref_ptr, igc_reloc_refs
 };
 
 /* Pointer type descriptors. */
@@ -151,6 +148,7 @@ gc_validate_spaces(gs_ref_memory_t **spaces, int max_space, gc_state_t *gcst)
 #else  /* !DEBUG */
 #  define end_phase(str) DO_NOTHING
 #endif /* DEBUG */
+
 void
 gs_gc_reclaim(vm_spaces * pspaces, bool global)
 {
@@ -374,6 +372,20 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
 	end_phase("finish trace");
     }
+
+#if NO_INVISIBLE_LEVELS
+    /* Filter save change lists with removing elements,
+       which point to unmarked blocks of refs. */
+    {
+	int i;
+
+	for_collected_spaces(i) {
+	    gs_ref_memory_t *mem = space_memories[i];
+
+	    alloc_save__filter_changes(mem);
+	}
+    }
+#endif
     /* Clear marks and relocation in spaces that are only being traced. */
     /* We have to clear the marks first, because we want the */
     /* relocation to wind up as o_untraced, not o_unmarked. */
