@@ -1,25 +1,21 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: siscale.c 2741 2002-06-16 03:58:15Z lpd $ */
+/* $Id: siscale.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Image scaling filters */
 #include "math_.h"
 #include "memory_.h"
 #include "stdio_.h"
-#include <assert.h>
 #include "gconfigv.h"
 #include "gdebug.h"
 #include "strimpl.h"
@@ -39,7 +35,7 @@
 
 typedef int PixelWeight;
 
-#  if arch_ints_are_short
+#  if ARCH_INTS_ARE_SHORT
 typedef long AccumTmp;
 #  else
 typedef int AccumTmp;
@@ -343,7 +339,7 @@ zoom_x(PixelTmp * tmp, const void /*PixelIn */ *src, int sizeofPixelIn,
 	if (sizeofPixelIn == 1) {
 	    zoom_x_loop(byte, int)
 	} else {		/* sizeofPixelIn == 2 */
-#if arch_ints_are_short
+#if ARCH_INTS_ARE_SHORT
 	    zoom_x_loop(bits16, long)
 #else
 	    zoom_x_loop(bits16, int)
@@ -570,9 +566,8 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
 
 	if (rleft == 0)
 	    return 0;		/* need more input */
-#ifdef DEBUG
-	assert(ss->src_y < ss->params.HeightIn);
-#endif
+	if (ss->src_y >= ss->params.HeightIn)
+	    return ERRC;
 	if (rleft >= rcount) {	/* We're going to fill up a row. */
 	    const byte *row;
 
