@@ -1,21 +1,17 @@
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
+  
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/* Copyright (C) 1996, 2000 Aladdin Enterprises.  All rights reserved.
-  
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zusparam.c 5215 2004-08-04 19:36:13Z stefan $ */
+/* $Id: zusparam.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* User and system parameter operators */
 #include "memory_.h"
 #include "string_.h"
@@ -198,7 +194,7 @@ private const bool_param_def_t system_bool_params[] =
 private void
 current_RealFormat(i_ctx_t *i_ctx_p, gs_param_string * pval)
 {
-#if arch_floats_are_IEEE
+#if ARCH_FLOATS_ARE_IEEE
     static const char *const rfs = "IEEE";
 #else
     static const char *const rfs = "not IEEE";
